<?php

/**
 *
 * $Id: class.Logger.php,v 1.11 2005/04/28 01:41:49 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     core
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.11 $
 *
 */

/**
 * coWiki - Logger. The Logger provides logging capabilities of debug
 * messages to the webserver error log.
 *
 * @package     core
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.3.4
 */
class Logger {

    static
        $bLoggerOff = true,
        $nMsgCount  = 0;

    const LOG_PARENT_CALLER = 1;

    // ---

    const D_OFF   = 0;
    const D_ERR   = 1;
    const D_WARN  = 2;
    const D_INFO  = 4;
    const D_SQL   = 8;
    const D_FORK  = 16;
    const D_IO    = 32;
    const D_EXE   = 64;

    const D_ALL   = 127;    // This is the sum of the above

    const D_REPT  = 16384;  // For internal use

    // --------------------------------------------------------------------
        
    protected function Logger() {}
    
    // This method is required to trigger the __autoload() machanism. After
    // this the internal constants are avaialable.
    public static function triggerAutoload() {}

    // --------------------------------------------------------------------

    public static function info($sStr, $nType = 0) {
        Logger::doLog(self::D_INFO, $sStr, $nType);
    }
    public static function information($sStr, $nType = 0) {
        Logger::doLog(self::D_INFO, $sStr, $nType);
    }

    public static function warn($sStr, $nType = 0) {
        Logger::doLog(self::D_WARN, $sStr, $nType);
    }
    public static function warning($sStr, $nType = 0) {
        Logger::doLog(self::D_WARN, $sStr, $nType);
    }

    public static function err($sStr, $nType = 0) {
        Logger::doLog(self::D_ERR, $sStr, $nType);
    }
    public static function error($sStr, $nType = 0) {
        Logger::doLog(self::D_ERR, $sStr, $nType);
    }

    public static function fork($sStr, $nType = 0) {
        Logger::doLog(self::D_FORK, $sStr, $nType);
    }
    public static function sql($sStr, $nType = 0) {
        Logger::doLog(self::D_SQL, $sStr, $nType);
    }
    public static function io($sStr, $nType = 0) {
        Logger::doLog(self::D_IO, $sStr, $nType);
    }

    public static function exe($sStr, $nType = 0) {
        Logger::doLog(self::D_EXE, $sStr, $nType);
    }
    public static function execute($sStr, $nType = 0) {
        Logger::doLog(self::D_EXE, $sStr, $nType);
    }

    public static function infowarn($sStr, $bFlag, $nType = 0) {
        if ($bFlag) {
            Logger::info($sStr, $nType);
        } else {
            Logger::warn($sStr, $nType);
        }
    }

    // --------------------------------------------------------------------

    private static function doLog($nMode, $sStr, $nType) {
        static
            $aBuf = array(),
            $nBufIdx = 0,
            $nLevel = null;

        // Until the Registry is not completely initialized we have a
        // chicken-egg problem here: we can not log as we do not know
        // what should be logged. Hence we buffer the stuff first and
        // output it if every object that we require is available.
        if (!class_exists('Registry') || !Registry::isInitialized()) {

            // Remember
            $aBuf[$nBufIdx]['STRN'] = Logger::prepareLogString($sStr, $nType);
            $aBuf[$nBufIdx]['MODE'] = $nMode;
            $aBuf[$nBufIdx]['PROF'] = function_exists('profile')
                                          ? profile() : '     ';
            $nBufIdx++;

            return;
        }

        // Check debug level, trick a bit with internal constants
        if ($nLevel == null) {
            $sConfValue = Registry::getInstance()->get('SOFTWARE_DEBUG_LEVEL');
            if ($sConfValue == '') { return; }
            
            $sConfValue = str_replace('D_', 'self::D_', $sConfValue);

            eval('$nLevel = '.$sConfValue.';');
        }

        // Do log?
        if (($nLevel & $nMode) == 0) { return; }

        self::$bLoggerOff = false;

        // ---

        // Log separator
        if (Logger::$nMsgCount == 0) {
            Logger::header();

            // If we have something buffered, output it now
            if ($nBufIdx > 0) {
                for ($i=0; $i<$nBufIdx; $i++) {
                
                    // Check debug level
                    if (($nLevel & $aBuf[$i]['MODE']) == 0) { continue; }

                    Logger::writeLogMessage(
                        $aBuf[$i]['STRN'],
                        $aBuf[$i]['MODE'],
                        $aBuf[$i]['PROF']
                    );
                }
            }
        }
        
        $sStr = Logger::prepareLogString($sStr, $nType);

        Logger::writeLogMessage(
            $sStr,
            $nMode,
            function_exists('profile') ? profile() : '     '
        );
    }

    // --------------------------------------------------------------------

    private static function prepareLogString($sStr, $nType) {

        // Get backtrace
        $aTrace = debug_backtrace();
        $sFile = 'n/a';
        $sLine = '?';

        $nIdx = 3;
        if ($nType == self::LOG_PARENT_CALLER) {
            $nIdx = 4;
        }

        if (!isset($aTrace[$nIdx]) || !isset($aTrace[$nIdx]['class'])) {
            $sFile = basename($aTrace[$nIdx-1]['file']);

            // Check if 'file' fits class/interface naming scheme:
            // 'class.TheNameOfTheClass.php'
            if (substr($sFile, 0, 6) == 'class.') {
                $sFile = substr($sFile, 6, -4);
            }

            $sLine = $aTrace[$nIdx-1]['line'];
        } else {
            $sFile = $aTrace[$nIdx]['class'];
            $sLine = $aTrace[$nIdx-1]['line'];
        }

        return $sFile . ' (' . $sLine . '): ' . $sStr;
    }

    // --------------------------------------------------------------------

    private static function writeLogMessage($sStr, $nMode, $nProfile) {
        static $nRep = 0, $sLog = '';

        $aDebug[self::D_ERR]  = 'ERR';
        $aDebug[self::D_WARN] = 'WRN';
        $aDebug[self::D_INFO] = '   ';
        $aDebug[self::D_SQL]  = 'SQL';
        $aDebug[self::D_FORK] = 'FRK';
        $aDebug[self::D_IO]   = 'I/O';
        $aDebug[self::D_EXE]  = 'EXE';
        $aDebug[self::D_REPT] = '-->';

        // Remove line feeds
        $sStr = str_replace("\n", '', $sStr);

        // Remove multiple spaces
        $sStr = preg_replace('# +#S', ' ', $sStr);

        // ---

        // Check for "Last message repeated"
        if ($sStr == $sLog) {
            $nRep++;
            return;
        } else {
            $sLog = $sStr;
        }

        if ($nRep != 0) {
            $sWarn = 'HEADS UP: Last message repeated ' . $nRep . ' time(s)';

            Logger::logMessage(
                $sWarn,
                ++Logger::$nMsgCount,
                $nProfile,
                $aDebug[self::D_REPT]
            );

            $sLog = '';
            $nRep = 0;
        }

        // Do not report empty strings
        if ($sStr == '') { return ''; }

        // ---

        $nPos = strpos($sStr, ':');

        if ($nPos) {
            $nPos += 2;

            $sMsg1 = substr($sStr, 0, $nPos);
            $sMsg2 = substr($sStr, $nPos);

            $sStr = $sMsg1 . wordwrap(
                                $sMsg2,
                                65 - $nPos, // Calc wrap width
                                "\n      |     | ".str_repeat(' ', $nPos),
                                true
                             );
        }

        Logger::logMessage(
            $sStr,
            ++Logger::$nMsgCount,
            $nProfile,
            $aDebug[$nMode]
        );
    }
    
    // --------------------------------------------------------------------

    private static function logMessage($sMsg, $nCount, $nProfile, $sPrefix) {
        error_log(sprintf('%s | %s | %s', $nProfile, $sPrefix, $sMsg));
    }

    // --------------------------------------------------------------------

    private static function header() {
        $sTime   = strftime('%a %d. %B %Y, %H:%M:%S', time());
        $sUri    = '['. $_SERVER['REQUEST_METHOD'] . ']' . chr(160)
                    . $_SERVER['REQUEST_URI'];
        $sRemote = gethostbyaddr($_SERVER['REMOTE_ADDR']) . chr(160)
                   . '(' . $_SERVER['REMOTE_ADDR'] . ')';

        error_log("\n" . ' Secs | Lvl | Message');
        error_log('------+-----+'.str_repeat('-', 66));
        error_log('      |     | Request time: ' . $sTime);
        error_log('      |     | Request URI: '  . $sUri);
        error_log('      |     | Requested by: ' . $sRemote);
    }

    // --------------------------------------------------------------------

    public static function shutdown() {
        static $bCalled = false;

        if (!self::$bLoggerOff && !$bCalled) {
            error_log('------+-----+'.str_repeat('-', 66));

            if (function_exists('profileTotal')) {
                error_log(profileTotal().' seconds - estimated total script'
                          .' execution time');
            }

            $bCalled = true;
        }
    }

    // --------------------------------------------------------------------

    // Self reflection: get debug constants, but remove private ones
    public function getDebugConstants() {
        $Ref = new ReflectionClass(__CLASS__);

        $aConst = $Ref->getConstants();
        foreach ($aConst as $k => $v) {

            // Remove private constants. The important ones for setting the
            // Logger behaviour will remain.
            switch ($k) {
                case 'LOG_PARENT_CALLER':
                case 'D_OFF':
                case 'D_ALL':
                case 'D_REPT':
                    unset($aConst[$k]);
                    break;
            }
        }

        return $aConst;        
    }

} // of class

?>